// BBO.h: interface for the CBBO class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BBO_H__590B78DF_C08E_45F3_B112_3F47A866E583__INCLUDED_)
#define AFX_BBO_H__590B78DF_C08E_45F3_B112_3F47A866E583__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "./EACommonClass/Individual.h"
#include "./EACommonClass/Rand.h"
#include "./EACommonClass/ProblemDef.h"

#define max_n 200		// only for EP

class CBBO  
{
public:
	CBBO();
	virtual ~CBBO();

	/* ------------------------class methods------------------------ */
public:
	// public methods
	/* initialize the decision variables of different test problems */
	void init_variables();							// initialize the decision variables

	/* initialize the population */
	void init_pop();								// initialize the population
	void init_pop_random();							// initialize the population randomly

	/* evaluate the individual and the population */
	void evaluate_ind(CIndividual &);				// evaluate the individual
	void evaluate_pop(CIndividual *, int);			// evaluate the population

	/* compare the two individuals */
	int  compare_ind(CIndividual *, CIndividual *);	// compare the two individuals
	void find_best_and_worst_index(CIndividual *, int);		// find the index of the best individual in the population

	/* execute the algorithms */
	void Run_Optimizer(int, double, int);			// execute the optimizer

	/* some complementary methods */
	void random_index(int *array_index, int all_size, int size);
	void bubble_sort(CIndividual* ,int *, int);		// bubble sort routine in ascending order
	void shell_sort(CIndividual* ,int *, int);		// shell sort routine in ascending order

	void display_result(int);						// display the results in the screen
	void report_result(int, ofstream &);			// report the results in the files
	void report_diversity(int, ofstream &);			// report the diversity in the files

	/* ------------------------class members------------------------ */	
public:
	// public attributes
	CIndividual *parent_pop;						// the main population
	CIndividual *child_pop;							// the child population
	int pop_size;									// size of the main population
	CIndividual best_individual;					// the current best individual
	int best_index;									// index of the current best individual
	int worst_index;
	CIndividual new_child;							// the new child generated by the genetic operator

	int flag_precision;								// stop criterion

	long int max_NFFEs;								// maximal number of fitness function evaluations
	long int max_iterations;						// maximal number of iterations
	long int evaluations;							// current number of fitness function evaluations
	long int gen;									// current generation counter
	int MINIMIZE;

	double *upper_bounds;							// upper bounds of the decision variables
	double *lower_bounds;							// lower bounds of the decision variables
	double known_optimal;							// known optima of the problems

	double pop_diversity;

	/* some other class objectives */
	double rnd_seed;
	CRand  m_rnd;

	int func_flag;									// flag of the function optimized
	CProblemDef	m_func;								// normal test functions

	int method_flag;

	/* Methods and attributes for the Biogeography-based Optimization (BBO) algorithm */
public:
	// Attributes
	double			p_modify;			// habitat modification probability
	double			p_mutate;			// habitat mutation probability
	int				elitism_size;		// size of the elitism strategy
	CIndividual		*elite;				// save the elitist solutions
	double			lambdaLower;		// lower bound for immigration probability per gene
	double			lambdaUpper;		// upper bound for immigration probability per gene
	double			dt;					// step size used for numerical integration of probabilities
	double			II;					// max immigration rate for each island
	double			EE;					// max emigration rate, for each island

	int				flag_prob;

	int				*sort_index;		// only for sorting the population

	double			*lambda;			// the immigration rate
	double			*mu;				// the emigration rate
	double			*prob;				// the species count probability
	double			*ProbDot;

	// Methods
	void			GetSpeciesCounts(CIndividual *, int);
	void			GetLambdaMu(CIndividual *, int);
	void			Population_Sort(CIndividual *, int);
	void			FeasibleFunction(CIndividual *pop, int size);
	int				IsEqual(CIndividual &indv1, CIndividual &indv2);
	void			ClearDups(CIndividual *pop, int size);

	double			f_max;
	double			f_min;

	void			run_BBO();
	void			run_BBO_EP();
};

#endif // !defined(AFX_BBO_H__590B78DF_C08E_45F3_B112_3F47A866E583__INCLUDED_)
